# Generated by Django 3.2.4 on 2021-07-02 11:47

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0004_serial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serial',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
        ),
    ]
