# Generated by Django 3.2.4 on 2021-07-01 15:08

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0002_auto_20210615_0942'),
    ]

    operations = [
        migrations.AlterField(
            model_name='choice',
            name='question',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='tests.question'),
        ),
        migrations.AlterField(
            model_name='question',
            name='question_text',
            field=models.CharField(default='how are you?', max_length=200, validators=[django.core.validators.MinLengthValidator(1)]),
        ),
    ]
